package at.ac.uibk.gitsearch.service;

import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.time.Duration;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.checkerframework.checker.nullness.qual.Nullable;
import org.codeability.sharing.plugins.api.ShoppingBasket;
import org.codeability.sharing.plugins.api.ShoppingBasket.ExerciseInfo;
import org.codeability.sharing.plugins.api.ShoppingBasket.UserInfo;
import org.gitlab4j.api.GitLabApi;
import org.gitlab4j.api.GitLabApiException;
import org.gitlab4j.api.ProjectApi;
import org.gitlab4j.api.RepositoryApi;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StreamUtils;
import org.springframework.util.StringUtils;

import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;

import at.ac.uibk.gitsearch.repository.gitlab.GitLabRepository;
import at.ac.uibk.gitsearch.security.jwt.TokenProvider.GitLabAccessInfo;
import at.ac.uibk.gitsearch.service.dto.SearchResultDTO;

/**
 * Service for exercise/course search results
 * <p>
 */
@Service
public class GitlabService {
	
	@Autowired
	private PluginManagementService pluginManagementService;
	@Autowired
	private GitLabRepository gitLabRepository;
	
	
		
		

	

	private final Logger log = LoggerFactory.getLogger(ShoppingBasketService.class);


    public Boolean repositoryExists(String projectID) {
        final GitLabApi gitLabApi = gitLabRepository.getGitLabApi(Optional.empty());
		final ProjectApi gitLabProjectApi = gitLabApi.getProjectApi();
		try{
			return gitLabProjectApi.getProject(projectID) != null;}
		catch(GitLabApiException e){
			log.error(e.getMessage(), e);
			return false;
		}
    }
    
	
	public InputStream getRepositoryZip(String exerciseID) throws GitLabApiException, IOException {
		
        final GitLabApi gitLabApi = gitLabRepository.getGitLabApi(Optional.empty());
		return rePackageGitLabProjectZip(new ZipInputStream(gitLabApi.getRepositoryApi().getRepositoryArchive(exerciseID, "HEAD", "zip")), "from project " + exerciseID);
	}
	
	/**
	 * chops of the main folder, and brings every file one level up. Also deleting
	 * all plain files in main folder.
	 * 
	 * @param zipIn the zip to be repackaged
	 * @param originalLocation the original location. For debug purposes only.
	 * @return
	 * @throws IOException
	 */
	protected InputStream rePackageGitLabProjectZip(ZipInputStream zipIn, String originalLocation) throws IOException {
		final PipedInputStream pis = new PipedInputStream(1024*256);
		final PipedOutputStream pos = new PipedOutputStream(pis);

		Runnable rePackage = () -> {
			try (ZipOutputStream zipOut = new ZipOutputStream(pos)) {

				ZipEntry zipInEntry = zipIn.getNextEntry();
				String prefix = "";
				if (zipInEntry.isDirectory()) { // main directory is prefix to all entries
					prefix = zipInEntry.getName();
				}

				while (zipInEntry != null) {
					if (zipInEntry.getName().startsWith(prefix)) {
						String newName = zipInEntry.getName().substring(prefix.length());
						if (!StringUtils.isEmpty(newName)) {
							ZipEntry zipOutEntry = new ZipEntry(newName);
							if (zipInEntry.isDirectory()) {
							log.debug("ignoring directory {}", zipInEntry.getName());
								zipOut.putNextEntry(zipOutEntry);
							} else {
								zipOut.putNextEntry(zipOutEntry);
								StreamUtils.copy(zipIn, zipOut);
								zipOut.closeEntry();
							}
						}}
					zipInEntry = zipIn.getNextEntry();
				}
				zipIn.close();
			} catch (IOException e) {
				log.error("Cannot rezip file from {}", originalLocation);
			}
		};

		new Thread(rePackage).start();

		return pis;

	}
	
	

}
